/**
 * University of Illinois/NCSA Open Source License
 *
 * Copyright (c) 2011 Milos Gligoric
 * All rights reserved.
 *
 * Developed by:  Milos Gligoric <milos.gligoric@gmail.edu>
 *                University of Illinois at Urbana-Champaign
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal with the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimers.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimers in the
 *    documentation and/or other materials provided with the distribution.
 *  * Neither the names of Milos Gligoric, University of Illinois at Urbana-
 *    Champaign, nor the names of its contributors may be used to endorse
 *    or promote products derived from this Software without specific prior
 *    written permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package codese;

import java.io.EOFException;
import java.io.IOException;
import java.io.ObjectInputStream;

import codese.extend.ASideValues;
import codese.extend.AGeneratedCode;

public final class CoDeSeInputStream extends ObjectInputStream {

  private static final String sideValuesClassName = "SV";
  private static final String topGeneratedClassName = "GeneratedCode";

  private String packageName;
  private ASideValues sideValues;
  private AGeneratedCode topGeneratedCode;
  private boolean firstRead;

  public static CoDeSeInputStream this$;

  @SuppressWarnings("static-access")
  public CoDeSeInputStream(String packageName) throws IOException {
    this.packageName = packageName;
    this.this$ = this;
    this.firstRead = true;
  }

  @Override
  public void defaultReadObject() throws IOException, ClassNotFoundException {
    // nop
  }

  private Class<?> sideValuesClass() throws Exception {
    return Class.forName(packageName + "." + sideValuesClassName);
  }

  private Class<?> topGeneratedClass() throws Exception {
    return Class.forName(packageName + "." + topGeneratedClassName);
  }

  @Override
  protected Object readObjectOverride() throws IOException {
    if (!firstRead) {
      return topGeneratedCode.getObject(sideValues.readObject());
    }
    firstRead = false;
    try {
      sideValues = (ASideValues) sideValuesClass().newInstance();
      topGeneratedCode = (AGeneratedCode) topGeneratedClass().newInstance();
    } catch (Exception e) {
      throw new IOException(e.getMessage());
    }
    return topGeneratedCode.genObjectGraph();
  }

  @Override
  public void reset() {
    firstRead = true;
    if (sideValues != null) {
      sideValues.reset();
    }
  }

  /**
   * Reads in a boolean.
   * 
   * @return the boolean read.
   * @throws EOFException
   *           If end of file is reached.
   * @throws IOException
   *           If other I/O error has occurred.
   */
  @Override
  public boolean readBoolean() throws IOException {
    return sideValues.readBoolean();
  }

  /**
   * Reads an 8 bit byte.
   * 
   * @return the 8 bit byte read.
   * @throws EOFException
   *           If end of file is reached.
   * @throws IOException
   *           If other I/O error has occurred.
   */
  @Override
  public byte readByte() throws IOException {
    return sideValues.readByte();
  }

  /**
   * Reads an unsigned 8 bit byte.
   * 
   * @return the 8 bit byte read.
   * @throws EOFException
   *           If end of file is reached.
   * @throws IOException
   *           If other I/O error has occurred.
   */
  @Override
  public int readUnsignedByte() throws IOException {
    return sideValues.readUnsignedByte();
  }

  /**
   * Reads a 16 bit char.
   * 
   * @return the 16 bit char read.
   * @throws EOFException
   *           If end of file is reached.
   * @throws IOException
   *           If other I/O error has occurred.
   */
  @Override
  public char readChar() throws IOException {
    return sideValues.readChar();
  }

  /**
   * Reads a 16 bit short.
   * 
   * @return the 16 bit short read.
   * @throws EOFException
   *           If end of file is reached.
   * @throws IOException
   *           If other I/O error has occurred.
   */
  @Override
  public short readShort() throws IOException {
    return sideValues.readShort();
  }

  /**
   * Reads an unsigned 16 bit short.
   * 
   * @return the 16 bit short read.
   * @throws EOFException
   *           If end of file is reached.
   * @throws IOException
   *           If other I/O error has occurred.
   */
  @Override
  public int readUnsignedShort() throws IOException {
    return sideValues.readUnsignedShort();
  }

  /**
   * Reads a 32 bit int.
   * 
   * @return the 32 bit integer read.
   * @throws EOFException
   *           If end of file is reached.
   * @throws IOException
   *           If other I/O error has occurred.
   */
  @Override
  public int readInt() throws IOException {
    return sideValues.readInt();
  }

  /**
   * Reads a 64 bit long.
   * 
   * @return the read 64 bit long.
   * @throws EOFException
   *           If end of file is reached.
   * @throws IOException
   *           If other I/O error has occurred.
   */
  @Override
  public long readLong() throws IOException {
    return sideValues.readLong();
  }

  /**
   * Reads a 32 bit float.
   * 
   * @return the 32 bit float read.
   * @throws EOFException
   *           If end of file is reached.
   * @throws IOException
   *           If other I/O error has occurred.
   */
  @Override
  public float readFloat() throws IOException {
    return sideValues.readFloat();
  }

  /**
   * Reads a 64 bit double.
   * 
   * @return the 64 bit double read.
   * @throws EOFException
   *           If end of file is reached.
   * @throws IOException
   *           If other I/O error has occurred.
   */
  @Override
  public double readDouble() throws IOException {
    return sideValues.readDouble();
  }

}
