/*****************************************************************************
 * University of Illinois/NCSA Open Source License
 *
 * Copyright (c) 2010 Rohan Sharma
 * All rights reserved.
 *
 * Developed by:  Rohan Sharma <sharma27@illinois.edu>
 *                University of Illinois at Urbana-Champaign
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal with the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimers.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimers in the
 *    documentation and/or other materials provided with the distribution.
 *  * Neither the names of Rohan Sharma, University of Illinois at Urbana-
 *    Champaign, nor the names of its contributors may be used to endorse
 *    or promote products derived from this Software without specific prior
 *    written permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *****************************************************************************/

package codese.dataset.intredblacktree;

import java.io.Serializable;

/**
 * A map implementation following the specification for the java.util.TreeMap
 * class but using primitive values of type "int" as keys.  The implementation
 * is an adaptation of red black trees described in the book "Introduction to
 * Algorithms" by Cormen, Leiserson, Rivest, and Stein.  The following code
 * can be used for testing research purposes.
 */
public class IntRedBlackTree implements Serializable {

  private static final long serialVersionUID = 1L;

  protected static final boolean RED = false;
  protected static final boolean BLACK = true;

  protected Node root;
  protected int size;

  public static class Node implements Serializable {
    private static final long serialVersionUID = 1L;

    int key;
    Object value;
    Node parent;
    Node left, right;
    boolean color;

    // public String toString() {
    //   // do not check cycles here
    //   String s = "N" + color + "[" + (left != null ? left.toString() : null)
    //     + "][" + (right != null ? right.toString() : null) + "]";
    //   return s;
    // }
    
    @Override
    public String toString() {
      String s = "{c:" + color + ";v" + value + "["
        + (left != null ? left.toString() : null) + "]["
        + (right != null ? right.toString() : null) + "]}";
      return s;
    }
  }

  protected Node parent(Node n) {
    return n == null ? null : n.parent;
  }

  protected Node left(Node n) {
    return n == null ? null : n.left;
  }

  protected Node right(Node n) {
    return n == null ? null : n.right;
  }

  public boolean containsKey(Integer key) {
    return findNode(key) != null;
  }

  public Object get(Integer key) {
    Node x = findNode(key);
    return x == null ? null : x.value;
  }

  public void put(Integer key, Object value) {
    Node x = new Node();
    x.key = key;
    x.value = value;
    treeInsert(x);
    size++;
  }

  public Object remove(Integer key) {
    Node ret = treeDelete(findNode(key));
    if (ret == null)
      return null;
    size--;
    return ret.value;
  }

  protected boolean getColor(Node x) {
    return x == null ? BLACK : x.color;
  }

  protected void setColor(Node x, boolean color) {
    if (x != null) {
      x.color = color;
    }
  }

  protected Node findNode(Integer key) {
    Node cur = root;
    while (cur != null && key.compareTo(cur.key) != 0)
      if (key.compareTo(cur.key) < 0)
        cur = cur.left;
      else
        cur = cur.right;
    return cur;
  }

  protected void leftRotate(Node x) {
    Node y = x.right;
    x.right = y.left;
    if (y.left != null)
      y.left.parent = x;
    y.parent = x.parent;
    if (x.parent == null)
      root = y;
    else if (x == x.parent.left)
      x.parent.left = y;
    else
      x.parent.right = y;
    y.left = x;
    x.parent = y;
  }

  protected void rightRotate(Node x) {
    Node y = x.left;
    x.left = y.right;
    if (y.right != null)
      y.right.parent = x;
    y.parent = x.parent;
    if (x.parent == null)
      root = y;
    else if (x == x.parent.right)
      x.parent.right = y;
    else
      x.parent.left = y;
    y.right = x;
    x.parent = y;
  }

  protected void treeInsert(Node z) {
    Node y = null;
    Node x = root;
    while (x != null) {
      y = x;
      if (((Integer) z.key).compareTo(x.key) < 0) {
        x = x.left;
        // insert duplicates to ensure the size of the structure
        // } else if (((Integer) z.key).compareTo(x.key) == 0) {
        // x.value = z.value;
        // size--;
        // return;
      } else {
        x = x.right;
      }
    }
    z.parent = y;
    if (y == null)
      root = z;
    else if (((Integer) z.key).compareTo(y.key) < 0)
      y.left = z;
    else
      y.right = z;
    z.left = null;
    z.right = null;
    z.color = RED;

    treeInsertFix(z);
  }

  protected void treeInsertFix(Node z) {
    while (getColor(z.parent) == RED) {
      if (parent(z) == left(parent(parent(z)))) {
        Node y = right(parent(parent(z)));
        if (getColor(y) == RED) {
          setColor(parent(z), BLACK);
          setColor(y, BLACK);
          setColor(parent(parent(z)), RED);
          z = parent(parent(z));
        } else {
          if (z == right(parent(z))) {
            z = parent(z);
            leftRotate(z);
          }
          setColor(parent(z), BLACK);
          setColor(parent(parent(z)), RED);
          if (parent(parent(z)) != null)
            rightRotate(parent(parent(z)));
        }
      } else {
        Node y = left(parent(parent(z)));
        if (getColor(y) == RED) {
          setColor(parent(z), BLACK);
          setColor(y, BLACK);
          setColor(parent(parent(z)), RED);
          z = parent(parent(z));
        } else {
          if (z == left(parent(z))) {
            z = parent(z);
            rightRotate(z);
          }
          setColor(parent(z), BLACK);
          setColor(parent(parent(z)), RED);
          if (parent(parent(z)) != null)
            leftRotate(parent(parent(z)));
        }
      }
    }
    root.color = BLACK;
  }

  protected Node treeDelete(Node z) {
    if (z == null)
      return null;
    Node x, y;
    if (z.left == null || z.right == null)
      y = z;
    else
      y = getIOS(z);
    if (y.left != null)
      x = y.left;
    else
      x = y.right;
    if (x != null)
      x.parent = y.parent;
    if (y.parent == null)
      root = x;
    else if (y == y.parent.left)
      y.parent.left = x;
    else
      y.parent.right = x;
    if (y != z) {
      z.key = y.key;
      z.value = y.value;
    }
    if (getColor(y) == BLACK)
      if (x == null)
        treeDeleteFix(y);
      else
        treeDeleteFix(x);
    return y;
  }

  protected void treeDeleteFix(Node x) {
    while (x.parent != null && getColor(x) == BLACK) {
      if (x == x.parent.left || x.parent.left == null) {
        Node w = x.parent.right;
        if (w == null)
          return;
        if (getColor(w) == RED) {
          w.color = BLACK;
          x.parent.color = RED;
          leftRotate(x.parent);
          w = x.parent.right;
        }
        if (getColor(w.left) == BLACK && getColor(w.right) == BLACK) {
          w.color = RED;
          x = x.parent;
        } else {
          if (getColor(w.right) == BLACK) {
            w.left.color = BLACK;
            rightRotate(w);
            w = x.parent.right;
          }
          w.color = x.parent.color;
          x.parent.color = BLACK;
          if (w.right != null)
            w.right.color = BLACK;
          leftRotate(x.parent);
          x = root;
        }
      } else {
        Node w = x.parent.left;
        if (w == null)
          return;
        if (getColor(w) == RED) {
          w.color = BLACK;
          x.parent.color = RED;
          rightRotate(x.parent);
          w = x.parent.left;
        }
        if (getColor(w.right) == BLACK && getColor(w.left) == BLACK) {
          w.color = RED;
          x = x.parent;
        } else {
          if (getColor(w.left) == BLACK) {
            w.right.color = BLACK;
            leftRotate(w);
            w = x.parent.left;
          }
          w.color = x.parent.color;
          x.parent.color = BLACK;
          if (w.left != null)
            w.left.color = BLACK;
          rightRotate(x.parent);
          x = root;
        }
      }
    }
    x.color = BLACK;
  }

  protected Node getIOS(Node z) {
    Node x, y = null;
    x = z.right;
    while (x != null) {
      y = x;
      x = x.left;
    }
    return y;
  }

  @Override 
  public String toString() {
    return root == null ? "null" : root.toString();
  }

}
